/*
 * compat.h
 * Copyright (C) 2005 Robin KAY. All rights reserved.
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 */

#ifndef COMPAT_H
#define COMPAT_H

#include <sys/types.h>

// Prevent the inclusion of <sys/stat_impl.h> under IA32
#ifdef __i386
#define _SYS_STAT_IMPL_H
#endif

#include <sys/time.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>
#include <fcntl.h>

// Define prototypes for functions found in <sys/stat_impl.h> under IA32 
// rather than the inlined functions supplied by default.
#ifdef __i386
extern int mknod(const char *, mode_t, dev_t);
extern int stat(const char *, struct stat *);
extern int lstat(const char *, struct stat *);
extern int _xmknod(const int, const char *, mode_t, dev_t);
extern int _xstat(const int, const char *, struct stat *);
extern int _lxstat(const int, const char *, struct stat *);
extern int _fxstat(const int, int, struct stat *);
#define fstat(fildes, buf) _fxstat(_STAT_VER, (fildes), (buf))
#endif // __i386

// Define constants and function prototypes introduced after Solaris 8
// under earlier releases.
#ifndef AT_FDCWD

// Define missing constants
#define AT_FDCWD 0xffd19553
#define AT_REMOVEDIR 0x1
#define F_GETXFL 45
#define O_XATTR 0x4000

// Define missing function prototypes
extern int openat(int, const char *, int, ...);
extern int renameat(int, const char *, int, const char *);
extern int unlinkat(int, const char *, int);
extern int futimesat(int, const char *, const struct timeval *);
#ifdef _ILP32
extern int openat64(int, const char *, int, ...);
#endif // _ILP32

#endif // AT_FDCWD

// Prototype for getdents64 is not provided in system headers
#ifdef _ILP32
int getdents64(int, dirent64_t *, size_t);
#endif // _ILP32

#endif
