/*
 * fsrewrite.c
 * Copyright (C) 2005 Robin KAY. All rights reserved.
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <unistd.h>

#define RULES_BUFSIZE_INCREMENT 128

static const char usage_string[] =
  "Usage: %s [-b base_path_of_rule [-t target_path ...] ...] command\n";

static const char rules_env_prefix[] = "FSREWRITE_RULES=";
static const char pwd_env_prefix[] = "FSREWRITE_PWD=";
static const char path_format[] = "%c%d%s";

int main(int argc, char **argv)
{
  char *rules_env, *rulesenv_buf, *pwdenv_buf;
  int rulesenv_len, rulesenv_bufsize, pwd_max_len;
  int seen_base_path;
  char chr;
  
  rules_env = getenv("FSREWRITE_RULES");
  if (!rules_env) {
    rules_env = "";
  }
  
  rulesenv_len = strlen(rules_env_prefix) + strlen(rules_env);
  rulesenv_bufsize = ((rulesenv_len / RULES_BUFSIZE_INCREMENT) + 1) * RULES_BUFSIZE_INCREMENT;
  rulesenv_buf = malloc(rulesenv_bufsize);
  strcpy(rulesenv_buf, rules_env_prefix);
  strcat(rulesenv_buf, rules_env);
  seen_base_path = 0;
  
  while ((chr = getopt(argc, argv, "b:t:")) != -1) {
    char dummy;
    int len;
    
    switch (chr) {
      case 'b':
      case 't': {
        if (chr == 'b') {
          seen_base_path = 1;
          chr = ':';
        }
        else {
          if (!seen_base_path) {
            fprintf(stderr, usage_string, argv[0]);
            return EXIT_FAILURE;
          } 
          chr = '+';
        }
        
        if (optarg[0] != '/') {
          fprintf(stderr, "%s: Error: Paths must be absolute.\n", argv[0]);
          return EXIT_FAILURE;
        }
        
        len = snprintf(&dummy, 1, path_format, chr, strlen(optarg), optarg);
        while (rulesenv_bufsize < rulesenv_len + len) {
          rulesenv_buf = realloc(rulesenv_buf, rulesenv_bufsize += RULES_BUFSIZE_INCREMENT);
        }
        sprintf(rulesenv_buf + rulesenv_len, path_format, chr, strlen(optarg), optarg);
        rulesenv_len += len;
        break;
      }
      case '?': {
        fprintf(stderr, usage_string, argv[0]);
        return EXIT_FAILURE;
      }
    }
  }
  
  if (optind >= argc) {
    fprintf(stderr, usage_string, argv[0]);
    return EXIT_FAILURE;
  }
  
  if (!seen_base_path) {
    fprintf(stderr, "%s: Warning: No rules have been specified.\n", argv[0]);
  }
  
  pwd_max_len = pathconf(".", _PC_PATH_MAX);
  pwdenv_buf = malloc(strlen(pwd_env_prefix) + pwd_max_len);
  strcpy(pwdenv_buf, pwd_env_prefix);
  getcwd(pwdenv_buf + strlen(pwd_env_prefix), pwd_max_len);
  
  putenv(rulesenv_buf);
  putenv(pwdenv_buf);
  putenv("LD_PRELOAD_32=" LIB_INST_PATH);
  putenv("LD_PRELOAD_64=" LIB64_INST_PATH);
  
  execvp(argv[optind], &(argv[optind]));
  perror(argv[0]);
  return EXIT_FAILURE;
}
