/*
 * mallory.h
 * Copyright (C) 2005 Robin KAY. All rights reserved.
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License, Version 1.0 only
 * (the "License").  You may not use this file except in compliance
 * with the License.
 */

#ifndef MALLORY_H
#define MALLORY_H

#include <sys/types.h>
#include <sys/stat.h>
#include <pthread.h>
#include <thread.h>

extern int is_mt;
extern char *rules_env, *pwd_env;

#define MUTEX_LOCK(m) if (is_mt) pthread_mutex_lock((m));
#define MUTEX_UNLOCK(m) if (is_mt) pthread_mutex_lock((m));

extern int (*libc_access)(const char *, int);
extern int (*libc_acl)(const char *, int, int, void *);
extern int (*libc_chdir)(const char *);
extern int (*libc_fchdir)(int);
extern int (*libc_chmod)(const char *, mode_t);
extern int (*libc_chown)(const char *, uid_t, gid_t);
extern int (*libc_lchown)(const char *, uid_t, gid_t);
extern int (*libc_close)(int);
extern int (*libc_dup)(int);
extern int (*libc_execve)(const char *, char *const *, char *const *);
extern int (*libc_fcntl)(int, int, ...);
extern char *(*libc_getcwd)(char *, size_t);
extern int (*libc_getdents)(int, void *, size_t);
extern int (*libc_link)(const char *, const char *);
extern int (*libc_mkdir)(const char *, mode_t);
extern int (*libc_mknod)(const char *, mode_t, dev_t);
extern int (*libc_open)(const char *, int, ...);
extern int (*libc_openat)(int, const char *, int, ...);
extern long (*libc_pathconf)(const char *, int);
extern ssize_t (*libc_readlink)(const char *, char *, size_t);
extern int (*libc_rename)(const char *, const char *);
extern int (*libc_renameat)(int, const char *, int, const char *);
extern int (*libc_resolvepath)(const char *, char *, size_t);
extern int (*libc_rmdir)(const char *);
extern int (*libc_stat)(const char *,  void *);
extern int (*libc_lstat)(const char *, void *);
extern int (*libc_statvfs)(const char *, void *);
extern int (*libc_symlink)(const char *, const char *);
extern int (*libc_unlink)(const char *);
extern int (*libc_unlinkat)(int, const char *, int);
extern int (*libc_utimes)(const char *, const void *);
extern int (*libc_futimesat)(int, const char *, const void *);

#ifdef __i386
extern int (*libc_xmknod)(const int, const char *, mode_t, dev_t);
extern int (*libc_xstat)(const int, const char *, struct stat *);
extern int (*libc_lxstat)(const int, const char *, struct stat *);
#endif

#ifdef _ILP32
extern int (*libc_getdents64)(int, void *, size_t);
extern int (*libc_open64)(const char *, int, ...);
extern int (*libc_openat64)(int, const char *, int, ...);
extern int (*libc_stat64)(const char *, void *);
extern int (*libc_lstat64)(const char *, void *);
extern int (*libc_statvfs64)(const char *, void *);
#endif

#endif
