#include <stdio.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <errno.h>
#include <fcntl.h>
#include <limits.h>
#include <time.h>
#include <unistd.h>

#define NUM_ITERATIONS 1000
#define NUM_FDS 4
#define BUF_SIZE (sizeof(dirent_t)+NAME_MAX)

int main(int argc, char **argv)
{
  int i;
  
  // Seed random number generator
  srand(time(NULL));
  
  for (i=0; i<NUM_ITERATIONS; i++) {
    char buf[2048];
    int fds[NUM_FDS];
    int j, ret;
    
    // Open a directory likely to have many entries
    if ((fds[0] = open("/", O_RDONLY)) < 0) {
      return EXIT_FAILURE;
    }
    
    // Duplicate file descriptor
    for (j=1; j<NUM_FDS; j++) {
      fds[j] = dup(fds[0]);
    }
    
    do {
      int fdidx, fdidx2, fd, size;
      
      // Pick a random file descriptor
      fdidx = rand() % 4;
      fd = fds[fdidx];
      size = sizeof(dirent_t);
      
      // Read a single directory entry
      do {
        ret = getdents(fds[0], (dirent_t *)buf, size);
      } while ((ret == -1) && (errno == EINVAL) && (size++));
      
      // Close the file descriptor and replace it with a duplicate of another
      close(fd);
      fdidx2 = rand() % 4;
      if (fdidx == fdidx2) {
        fdidx2 = (fdidx2 + 1) % 4;
      }
      fds[fdidx] = dup(fds[fdidx2]);
    } while (ret > 0);
    
    // Return an error if getdents fails 
    if (ret == -1) {
      return EXIT_FAILURE;
    }
    
    // Close all the file descriptors
    for (j=0; j<NUM_FDS; j++) {
      close(fds[j]);
    }
  }
  
  return EXIT_SUCCESS;
}
